package audio;

import audio.RawWaveUtils;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Random;

class RawWaveUtils {

  protected static boolean saveSamplesToDisk(byte[] samples, String pathFileName) {
    // file su disco
    File file = new File(pathFileName);
    if (!file.exists()) {
      try {
        file.createNewFile();
      } catch (IOException ex) {
        return false;
      }
    }
    try {
      FileOutputStream fos = new FileOutputStream(file);
      fos.write(samples);
      fos.close();
    } catch (IOException ex) {
      return false;
    }
    return true;
  }
}

class RawWave8BitUtils extends RawWaveUtils {

  public static byte[] createWhiteNoise(int nSamples) {
    byte[] result = new byte[nSamples];
    Random rand = new Random();
    for (int i = 0; i < nSamples; i++) {
      result[i] = (byte) (rand.nextInt(256));
    }
    return result;
  }

  public static boolean save(byte[] samples, String pathFileName) {
    return saveSamplesToDisk(samples, pathFileName);
  }
}

class RawWave16BitUtils extends RawWaveUtils {

  public static short[] createWhiteNoise(int nSamples) {
    short[] result = new short[nSamples];
    Random rand = new Random();
    for (int i = 0; i < nSamples; i++) {
      result[i] = (short) (rand.nextInt(65535) - 32768);
    }
    return result;
  }

  public static boolean save(short[] samples, ByteOrder endianess, String pathFileName) {
    // conversione dell'array di short in array di byte
    byte[] bytes = new byte[samples.length * 2];
    ByteBuffer.wrap(bytes).order(endianess).asShortBuffer().put(samples);
    return saveSamplesToDisk(bytes, pathFileName);
  }
}

public class SaveRaw {

  public static void main(String[] args) {

    short[] mySamples16 = {1, 2, 3, 4, 5};
    RawWave16BitUtils.save(mySamples16, ByteOrder.BIG_ENDIAN, "D:/fake_big_16.raw");
    RawWave16BitUtils.save(mySamples16, ByteOrder.LITTLE_ENDIAN, "D:/fake_little_16.raw");
    byte[] mySamples8 = {1, 2, 3, 4, 5};
    RawWave8BitUtils.save(mySamples8, "D:/fake_8.raw");
    // verifica scrittura di valori rispetto all'endianness con editor esadecimale

    int nSamples = 44100;
    byte[] whiteNoise8bit = RawWave8BitUtils.createWhiteNoise(nSamples);
    RawWave8BitUtils.save(whiteNoise8bit, "D:/white.raw");
  }
}
